using System;
using UnityEngine;
using UnityEngine.Internal;

using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Threading;
using MVR;
using MeshVR;
using System.Linq;
using GPUTools.Hair.Scripts.Runtime.Render;
using AssetBundles;
using System.Reflection;

namespace Stopper {
	public class CrossGenderHair : MVRScript {

		DAZCharacterSelector selector;

		JSONStorableBool enableMaleHairOnFemale;
		JSONStorableBool enableFemaleHairOnMale;

		public override void Init() {

			selector = containingAtom.GetStorableByID("geometry") as DAZCharacterSelector;

			
			enableMaleHairOnFemale = new JSONStorableBool("Enable Male Hair on Female", true);
			RegisterBool(enableMaleHairOnFemale);
			CreateToggle(enableMaleHairOnFemale, false);

			enableFemaleHairOnMale = new JSONStorableBool("Enable Female Hair on Male", true);
			RegisterBool(enableFemaleHairOnMale);
			CreateToggle(enableFemaleHairOnMale, false);
	
			var goButton = CreateButton("Go!", false);
			goButton.button.onClick.AddListener(() => {
				Go();
			});

			return;
		}

		private void Go(){
			EnableFemaleHairOnMale();
			EnableMaleHairOnFemale();
			
			selector.InitHairItems();
			selector.hairSelectorMaleUI.Resync();
		}

		private void EnableFemaleHairOnMale(){
			if(!enableFemaleHairOnMale.val) {
				return;
			}

			HashSet<string> maleHairs = new HashSet<string>();
			foreach(var hair in selector.maleHairItems) {
				maleHairs.Add(hair.displayName);
			}

			foreach(var hair in selector.femaleHairItems) {
				if(hair.name == "CustomHairItem" && !maleHairs.Contains(hair.displayName)) {
					var newHair = UnityEngine.Object.Instantiate(hair, selector.maleHairContainer);
					try {
						newHair.skin = selector.selectedCharacter.skin;
					} catch (Exception e) {
						// eat
					}
				}
			}
		}

		private void EnableMaleHairOnFemale(){
			if(!enableMaleHairOnFemale.val) {
				return;
			}

			HashSet<string> femaleHairs = new HashSet<string>();
			foreach(var hair in selector.femaleHairItems) {
				femaleHairs.Add(hair.displayName);
			}

			foreach(var hair in selector.maleHairItems) {
				if(hair.name == "CustomHairItem" && !femaleHairs.Contains(hair.displayName)) {
					var newHair = UnityEngine.Object.Instantiate(hair, selector.femaleHairContainer);
					try {
						newHair.skin = selector.selectedCharacter.skin;
					} catch (Exception e) {
						// eat
					}
				}
			}
		}

		public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true) {
			base.RestoreFromJSON(jc,restorePhysical,restoreAppearance,presetAtoms,false);

			Go();
			containingAtom.RestoreFromLast(selector);
		}
    }
}
